#!/usr/bin/env python3

#
# Description
#   This is a command line tool for show simulation log smarter.
#   The smartlog has many interactive features, such as warning/error
#   filter, display the first error, edit rtl at place, etc.

#
# License
#   This tool is published with GNU GPL v3.0 License.
#

#
# Author
#   chenfeng
#

#
# History
#   2018-12-26 optimize logic
#   2018-12-24 initial version
#

import os
import sys
import re
import colorama

editor = 'emacs' #emacs | gvim
logfile = 'sim.log'
msglist = []
sellist = []
progress = 'all'
typ = 'all'

def log_parse(logfile):
    global msglist
    with open(logfile, 'r') as f:
        lines = f.readlines()
        #parse ncvlog: *W|E
        for i in range(len(lines)):
            match = re.search(r'(ncvlog|ncelab):\s\*([WE]),\w+\s\(([^,]+),(\d+)', lines[i])
            if match:
                msg = {}
                msg['file'] = match.group(3)
                msg['code'] = lines[i-2].rstrip('\n')
                msg['position'] = lines[i-1].rstrip('\n')
                msg['detail'] = lines[i].rstrip('\n')
                if match.group(2) == 'W':
                    msg['typ'] = 'warn'
                else:
                    msg['typ'] = 'err'
                msg['lineno'] = match.group(4)
                if match.group(1) == 'ncvlog':
                    msg['progress'] = 'compile'
                elif match.group(1) == 'ncelab':
                    msg['progress'] = 'elab'
                msglist.append(msg)

# select * from msglist where progress = ? and typ = ?
def select_msg():
    global sellist
    sellist.clear() #clear
    for msg in msglist:
        if progress == 'all' and typ == 'all':
            sellist.append(msg)
        elif progress == 'all' and typ != 'all':
            if msg['typ'] == typ:
                sellist.append(msg)
        elif progress != 'all' and typ == 'all':
            if msg['process'] == progress:
                sellist.append(msg)
        else:
            if msg['process'] == progress and msg['typ'] == typ:
                sellist.append(msg)

#index : index of msglist
def print_one_msg(index):
    pos = index
    
    msg = sellist[index]

    #print(colorama.Back.BLACK, end='')
    #print(colorama.Fore.WHITE, end='')
    print('== {}: {} {}/{} =='.format(progress, typ, index, len(sellist)).center(40, '='))
    #print(colorama.Style.RESET_ALL)
    #file
    print(colorama.Fore.BLUE + msg['file'] + " +" + msg['lineno'])
    print(colorama.Style.RESET_ALL)
    #code
    print(msg['code']) 
    print(msg['position'])
    if msg['typ'] == 'warn':
        print(colorama.Fore.MAGENTA + msg['detail'])
    else:
        print(colorama.Fore.RED + msg['detail'])
        print(colorama.Style.RESET_ALL, end='')
    return pos+1


if __name__ == '__main__':
    log_parse(logfile)

    pos = 0
    errno = 0
    warnno = 0
    
    #display one error if exists
    progress = 'all'
    typ = 'err'
    select_msg()
    errno = len(sellist)
    if errno > 0:
        pos = print_one_msg(0)

    #display more error if exists
    while pos < errno:
        c = input('q:exit, n:next, p=a|c|e, t=a|e|w, edit > ')
        if c == 'q':
            break
        elif c == 'n' or c == '':
            pos = print_one_msg(pos)
        elif c == 'p=a':
            progress = 'all'
            select_msg()
        elif c == 'p=c':
            progress = 'compile'
            select_msg()
        elif c == 'p=e':
            progress = 'elab'
            select_msg()
        elif c == 't=a':
            typ = 'all'
            select_msg()
        elif c == 't=e':
            typ = 'err'
            select_msg()
        elif c == 't=w':
            typ = 'warn'
            select_msg()
        elif c == 'edit':
            os.system('{} +{} {}'.format(editor,
                                         msglist[pos - 1]['lineno'],
                                         msglist[pos - 1]['file']))
            os._exit(0)

            
